import 'package:dynamic_timeline/dynamic_timeline.dart';
import 'package:dynamic_timeline_example/widgets/widgets.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';

class GanttChart extends StatefulWidget {
  const GanttChart({Key? key}) : super(key: key);

  @override
  State<GanttChart> createState() => _GanttChartState();
}

class _GanttChartState extends State<GanttChart> {
  late final ScrollController scrollController;
  final items = [
    TimelineItem(
      startDateTime: DateTime(2022, 1, 1),
      endDateTime: DateTime(2022, 1, 15),
      child: const Event(title: 'Event 1'),
    ),
    TimelineItem(
      startDateTime: DateTime(2022, 1, 20),
      endDateTime: DateTime(2022, 2, 1),
      position: 1,
      child: const Event(title: 'Event 2'),
    ),
    TimelineItem(
      startDateTime: DateTime(2022, 2, 1),
      endDateTime: DateTime(2022, 3, 15),
      position: 2,
      child: const Event(title: 'Event 3'),
    ),
  ];

  @override
  void initState() {
    super.initState();
    scrollController = ScrollController();
  }

  @override
  void dispose() {
    scrollController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(title: const Text('Gantt Chart')),
      body: Center(
        child: Padding(
          padding: const EdgeInsets.all(40),
          child: Scrollbar(
            controller: scrollController,
            isAlwaysShown: true,
            child: SingleChildScrollView(
              scrollDirection: Axis.horizontal,
              controller: scrollController,
              child: DynamicTimeline(
                firstDateTime: DateTime(2022, 1, 1),
                lastDateTime: DateTime(2022, 12, 31),
                labelBuilder: DateFormat('dd').format,
                axis: Axis.horizontal,
                intervalDuration: const Duration(days: 1),
                crossAxisCount: 3,
                intervalExtent: 20,
                children: items,
              ),
            ),
          ),
        ),
      ),
    );
  }
}
